@@ -730,8 +730,9 @@ module Rubinius
         pos(g)
 
         g.push_rubinius
+        g.find_const :Type
         g.swap
-        g.send :object_metaclass, 1
+        g.send :object_singleton_class, 1
 
         if @body
           # if @body just returns self, don't bother with it.
@@ -773,6 +774,10 @@ module Rubinius
         @name = :__eval_script__
       end
 
+      def should_cache?
+        !@body.kind_of?(AST::ClosedScope)
+      end
+
       def search_scopes(name)
         depth = 1
         scope = @variable_scope
@@ -805,7 +810,6 @@ module Rubinius
 
       def new_local(name)
         variable = Compiler::EvalLocalVariable.new name
-        @variable_scope.dynamic_locals[name] = nil
         variables[name] = variable
       end
 
