@@ -79,7 +79,7 @@ module Rubinius
 
         if @vcall_style
           g.push :true
-          g.send :__respond_to_eh__, 2
+          g.send :__respond_to_p__, 2
         else
           g.push_self
           g.invoke_primitive :vm_check_callable, 3
@@ -260,7 +260,7 @@ module Rubinius
       end
 
       def splat?
-        @arguments.kind_of? SplatValue
+        @arguments.kind_of? SplatValue or @arguments.kind_of? ConcatArgs
       end
 
       def bytecode(g)
@@ -480,6 +480,7 @@ module Rubinius
       def initialize(line, arguments)
         @line = line
         @optional = 0
+        @arguments = nil
 
         @splat_index = -1
         @required_args = 0
