@@ -3,8 +3,9 @@ module Rubinius
     :which_ruby     => :ruby,
     :build_ruby     => "/usr/local/bin/ruby18",
     :build_rake     => "/usr/local/bin/rake18",
-    :llvm           => :config,
-    :llvm_configure => "/usr/local/bin/llvm-config",
+    :build_perl     => "perl",
+    :llvm           => :no,
+    :llvm_configure => "",
     :cc             => "cc",
     :cxx            => "g++",
     :user_cflags    => "-O2 -pipe",
@@ -12,15 +13,16 @@ module Rubinius
     :user_ldflags   => "-L/usr/local/lib",
     :defines        => ["HAS_READLINE"],
     :curses         => "curses",
-    :host           => "i386-unknown-openbsd4.9",
-    :cpu            => "i386",
+    :host           => "alpha-unknown-openbsd5.3",
+    :cpu            => "alpha",
     :vendor         => "unknown",
-    :os             => "openbsd4.9",
+    :os             => "openbsd",
     :little_endian  => true,
-    :sizeof_long    => 4,
+    :sizeof_long    => 8,
     :bindir         => "/usr/local/lib/rubinius/bin",
     :libdir         => "/usr/local/lib/rubinius",
     :runtime        => "/usr/local/lib/rubinius/runtime",
+    :kernel_path    => "/usr/local/lib/rubinius/kernel",
     :lib_path       => "/usr/local/lib/rubinius/lib",
     :ext_path       => "/usr/local/lib/rubinius/lib/ext",
     :includedir     => "/usr/local/lib/rubinius/include",
@@ -29,9 +31,9 @@ module Rubinius
     :sitedir        => "/usr/local/lib/rubinius/site",
     :vendordir      => "/usr/local/lib/rubinius/vendor",
     :program_name   => "rbx",
-    :version        => "1.2.0",
+    :version        => "1.2.4",
     :libversion     => "1.2",
-    :release_date   => "2010-12-21",
-    :config_version => 27
+    :release_date   => "2011-07-05",
+    :config_version => 38
   }
 end
