@@ -108,7 +108,7 @@ class BasicSocket < IO
   end
 
   def close_read
-    ensure_open_and_readable
+    ensure_open
 
     # If we were only in readonly mode, close it all together
     if @mode & ACCMODE == RDONLY
@@ -118,13 +118,13 @@ class BasicSocket < IO
     # MRI doesn't check if shutdown worked, so we don't.
     Socket::Foreign.shutdown @descriptor, 0
 
-    @mode = @mode & ~RDONLY
+    @mode = WRONLY
 
     nil
   end
 
   def close_write
-    ensure_open_and_writable
+    ensure_open
 
     # If we were only in writeonly mode, close it all together
     if @mode & ACCMODE == WRONLY
@@ -133,8 +133,8 @@ class BasicSocket < IO
 
     Socket::Foreign.shutdown @descriptor, 1
 
-    # Remove write from the mode bits
-    @mode = @mode & ~WRONLY
+    # Mark it as read only
+    @mode = RDONLY
 
     nil
   end
@@ -146,14 +146,15 @@ class BasicSocket < IO
   #
   def recv_nonblock(bytes_to_read, flags = 0)
     fcntl Fcntl::F_SETFL, Fcntl::O_NONBLOCK
-    recvfrom bytes_to_read, flags
-
+    socket_recv bytes_to_read, flags, 0
   rescue Errno::EWOULDBLOCK
     raise Errno::EAGAIN
   end
 
-
-#
+  def shutdown(how = 2)
+    err = Socket::Foreign.shutdown @descriptor, how
+    Errno.handle "shutdown" unless err == 0
+  end
 
 end
 
@@ -200,8 +201,8 @@ class Socket < BasicSocket
     attach_function :shutdown, [:int, :int], :int
     attach_function :listen,   [:int, :int], :int
     attach_function :socket,   [:int, :int, :int], :int
-    attach_function :send,     [:int, :pointer, :int, :int], :int
-    attach_function :recv,     [:int, :pointer, :int, :int], :int
+    attach_function :send,     [:int, :pointer, :size_t, :int], :int
+    attach_function :recv,     [:int, :pointer, :size_t, :int], :int
     attach_function :recvfrom, [:int, :pointer, :size_t, :int,
                                 :pointer, :pointer], :int
 
@@ -741,7 +742,7 @@ class Socket < BasicSocket
     if status < 0
       begin
         Errno.handle "connect(2)"
-      rescue Errno::ISCONN
+      rescue Errno::EISCONN
         return 0
       end
     end
@@ -762,6 +763,8 @@ class Socket < BasicSocket
 end
 
 class UNIXSocket < BasicSocket
+  include IO::TransferIO
+
   # Coding to the lowest standard here.
   def recvfrom(bytes_read, flags = 0)
     # FIXME 2 is hardcoded knowledge from io.cpp
@@ -793,9 +796,9 @@ class UNIXSocket < BasicSocket
     phase = 'socket(2)'
     sock = Socket::Foreign.socket Socket::Constants::AF_UNIX, Socket::Constants::SOCK_STREAM, 0
 
-    IO.setup self, sock, 'r+', true
+    Errno.handle phase if sock < 0
 
-    Errno.handle phase if descriptor < 0
+    IO.setup self, sock, 'r+', true
 
     sockaddr = Socket.pack_sockaddr_un(@path)
 
@@ -808,14 +811,17 @@ class UNIXSocket < BasicSocket
     end
 
     if status < 0 then
-      Socket::Foreign.close descriptor
+      close
       Errno.handle phase
     end
 
     if server then
       phase = 'listen(2)'
       status = Socket::Foreign.listen descriptor, 5
-      Errno.handle phase if status < 0
+      if status < 0
+        close
+        Errno.handle phase
+      end
     end
 
     return sock
@@ -834,6 +840,18 @@ class UNIXSocket < BasicSocket
     ["AF_UNIX", sock_path]
   end
 
+  def recv_io(klass=IO, mode=nil)
+    begin
+      fd = recv_fd
+    rescue PrimitiveFailure
+      raise SocketError, "file descriptor was not passed"
+    end
+
+    return fd unless klass
+
+    klass.for_fd(fd, mode)
+  end
+
   class << self
     def socketpair(type=Socket::SOCK_STREAM, protocol=0)
       Socket.socketpair(Socket::PF_UNIX, type, protocol, self)
@@ -925,7 +943,6 @@ class UDPSocket < IPSocket
 
     if status < 0
       Errno.handle 'bind(2)'
-      Socket::Foreign.close descriptor
     end
 
     status
@@ -938,7 +955,6 @@ class UDPSocket < IPSocket
     status = Socket::Foreign.connect descriptor, sockaddr
 
     if status < 0
-      Socket::Foreign.close descriptor
       Errno.handle syscall
     end
 
@@ -992,11 +1008,11 @@ class TCPSocket < IPSocket
   end
 
 
-  def initialize(host, port)
+  def initialize(host, port, local_host=nil, local_service=nil)
     @host = host
     @port = port
 
-    tcp_setup @host, @port
+    tcp_setup @host, @port, local_host, local_service
   end
 
   def tcp_setup(remote_host, remote_service, local_host = nil,
@@ -1013,59 +1029,88 @@ class TCPSocket < IPSocket
                                                    Socket::SOCK_STREAM, 0,
                                                    flags)
 
-    if server == false and (local_host or local_service) then
+    if server == false and (local_host or local_service)
+      local_host    = local_host.to_s    if local_host
+      local_service = local_service.to_s if local_service
       @local_addrinfo = Socket::Foreign.getaddrinfo(local_host,
                                                     local_service,
                                                     Socket::AF_UNSPEC,
                                                     Socket::SOCK_STREAM, 0, 0)
     end
 
+    sock = nil
+
     @remote_addrinfo.each do |addrinfo|
       flags, family, socket_type, protocol, sockaddr, canonname = addrinfo
 
-      status = Socket::Foreign.socket family, socket_type, protocol
+      sock = Socket::Foreign.socket family, socket_type, protocol
       syscall = 'socket(2)'
-      IO.setup self, status, nil, true
 
-      next if descriptor < 0
+      next if sock < 0
 
-      if server then
-        status = 1
-
-        begin
-          setsockopt(Socket::Constants::SOL_SOCKET,
-                     Socket::Constants::SO_REUSEADDR, true)
-        rescue SystemCallError
+      if server
+        FFI::MemoryPointer.new :socklen_t do |val|
+          val.write_int 1
+          level = Socket::Constants::SOL_SOCKET
+          optname = Socket::Constants::SO_REUSEADDR
+          error = Socket::Foreign.setsockopt(sock, level,
+                                             optname, val,
+                                             val.total)
+          # Don't check error because if this fails, we just continue
+          # anyway.
         end
 
-        status = Socket::Foreign.bind descriptor, sockaddr
+        status = Socket::Foreign.bind sock, sockaddr
         syscall = 'bind(2)'
       else
-        if @local_addrinfo then
-          status = bind descriptor, @local_addrinfo.first[4]
-          syscall = 'bind(2)'
+        if @local_addrinfo
+          # Pick a local_addrinfo for the family and type of
+          # the remote side
+          li = @local_addrinfo.find do |i|
+            i[1] == family && i[2] == socket_type
+          end
+
+          if li
+            status = Socket::Foreign.bind sock, li[4]
+            syscall = 'bind(2)'
+          else
+            status = 1
+          end
+        else
+          status = 1
         end
 
-        if status >= 0 then
-          status = Socket::Foreign.connect descriptor, sockaddr
+        if status >= 0
+          status = Socket::Foreign.connect sock, sockaddr
           syscall = 'connect(2)'
         end
       end
 
-      break if status >= 0
+      if status < 0
+        Socket::Foreign.close sock
+      else
+        break
+      end
     end
 
     if status < 0
       Errno.handle syscall
-      Socket::Foreign.close descriptor
     end
 
-    if server then
-      err = Socket::Foreign.listen descriptor, 5
-      Errno.handle syscall unless err == 0
+    if server
+      err = Socket::Foreign.listen sock, 5
+      unless err == 0
+        Socket::Foreign.close sock
+        Errno.handle syscall
+      end
     end
 
-    IO.setup self, descriptor, nil, true
+    # Only setup once we have found a socket we can use. Otherwise
+    # because we manually close a socket fd, we can create an IO fd
+    # alias condition which causes EBADF because when an IO is finalized
+    # and it's fd has been closed underneith it, we close someone elses
+    # fd!
+    IO.setup self, sock, nil, true
   end
   private :tcp_setup
 
@@ -1086,6 +1131,11 @@ class TCPServer < TCPSocket
       host = nil
     end
 
+    if String === host and port.nil? then
+      port = Integer(host)
+      host = nil
+    end
+
     port = StringValue port unless port.kind_of? Fixnum
 
     @host = host
